# Generated by Django 3.1.2 on 2021-03-29 09:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0005_auto_20210326_1628'),
    ]

    operations = [
        migrations.AddField(
            model_name='reductionrun',
            name='reduction_host',
            field=models.TextField(blank=True, default='', verbose_name='Reduction hostname'),
        ),
    ]



# Generated by Django 3.1.2 on 2021-02-09 15:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('reduction_viewer', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Variable',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('value', models.CharField(max_length=300)),
                ('type', models.CharField(max_length=50)),
                ('is_advanced', models.BooleanField(default=False)),
                ('help_text', models.TextField(blank=True, default='', null=True)),
            ],
        ),
        migrations.CreateModel(
            name='RunVariable',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reduction_run', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='run_variables', to='reduction_viewer.reductionrun')),
                ('variable', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='runs', to='instrument.variable')),
            ],
        ),
        migrations.CreateModel(
            name='InstrumentVariable',
            fields=[
                ('variable_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='instrument.variable')),
                ('experiment_reference', models.IntegerField(blank=True, null=True)),
                ('start_run', models.IntegerField(blank=True, null=True)),
                ('tracks_script', models.BooleanField(default=False)),
                ('instrument', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='reduction_viewer.instrument')),
            ],
            bases=('instrument.variable',),
        ),
    ]



# Generated by Django 3.1.2 on 2021-03-26 16:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0004_instrument_is_flat_output'),
    ]

    operations = [
        migrations.AlterField(
            model_name='experiment',
            name='reference_number',
            field=models.IntegerField(unique=True),
        ),
    ]



"""
Django settings for autoreduce_db project.

Generated by 'django-admin startproject' using Django 3.2.

For more information on this file, see
https://docs.djangoproject.com/en/3.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.2/ref/settings/
"""

import os
from autoreduce_utils.credentials import DB_CREDENTIALS
from autoreduce_utils.settings import PROJECT_DEV_ROOT

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-pd8%8fgsg*fo#0jvi9@0eh*+i(+vtaou2q@588cjr3=x5+$-r7'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition
INSTALLED_APPS = [  # Minimal apps required to setup JUST the ORM - (increases ORM setup speed)
    'autoreduce_db.reduction_viewer',
    'autoreduce_db.instrument',
]

# Database
# https://docs.djangoproject.com/en/3.2/ref/settings/#databases

if "RUNNING_VIA_PYTEST" in os.environ:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:',
        }
    }
elif "AUTOREDUCTION_PRODUCTION" in os.environ:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': DB_CREDENTIALS.database,
            'USER': DB_CREDENTIALS.username,
            'PASSWORD': DB_CREDENTIALS.password,
            'HOST': DB_CREDENTIALS.host,
            'PORT': DB_CREDENTIALS.port,
            'OPTIONS': {
                'init_command': "SET sql_mode='STRICT_TRANS_TABLES'",
            },
        }
    }

else:  # the default development DB backend
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(PROJECT_DEV_ROOT, "sqlite3.db"),
        }
    }



# Generated by Django 3.1.2 on 2021-03-10 16:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('reduction_viewer', '0002_remove_reductionrun_cancel'),
    ]

    operations = [
        migrations.RenameField(
            model_name='reductionrun',
            old_name='run_name',
            new_name='run_description',
        ),
    ]



# ############################################################################### #
# Autoreduction Repository : https://github.com/ISISScientificComputing/autoreduce
#
# Copyright &copy; 2019 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""
Models that represent the tables in the database
"""
from django.core.validators import MinValueValidator, MaxLengthValidator
from django.db import models


class Instrument(models.Model):
    """
    Hold data about an Instrument
    """
    name = models.CharField(max_length=80)
    is_active = models.BooleanField(default=False)
    is_paused = models.BooleanField(default=False)
    is_flat_output = models.BooleanField(default=False)

    def __str__(self):
        """
        :return: str representation of instrument
        """
        return f"{self.name}"

    def get_last_for_rerun(self, runs=None):
        """
        Gets the last non-skipped run, unless all the runs are skipped.
        In that case it just gets the last skipped run.
        """
        if not runs:
            runs = self.reduction_runs.all()

        # try to get the last non-skipped run
        last_run = runs.exclude(status=Status.get_skipped()).last()
        if not last_run:
            # if no non-skipped runs exist, just return the last skipped one
            last_run = runs.last()
        return last_run


class Experiment(models.Model):
    """
    Holds data about an Experiment
    """
    reference_number = models.IntegerField(unique=True)

    def __str__(self):
        """
        :return: str representation of experiment
        """
        return f"RB{self.reference_number}"


class Status(models.Model):
    """
    Enum table for status types of messages
    """
    _cached_statuses = {}
    STATUS_CHOICES = (('q', 'Queued'), ('p', 'Processing'), ('s', 'Skipped'), ('c', 'Completed'), ('e', 'Error'))

    value = models.CharField(max_length=1, choices=STATUS_CHOICES)

    def value_verbose(self):
        """
        :return: (str) the status as its textual value
        """
        return dict(Status.STATUS_CHOICES)[self.value]

    def __str__(self) -> str:
        """
        :return: str representation of status
        """
        return self.value_verbose()

    @staticmethod
    def _get_status(status_value: str):
        """
        Attempt to get a status matching the given name or create one if it
        doesn't yet exist
        :param status_value: The value of the status record in the database
        """
        if status_value in Status._cached_statuses:
            return Status._cached_statuses[status_value]
        else:
            status_record = Status.objects.get_or_create(value=status_value)[0]
            Status._cached_statuses[status_value] = status_record
        return status_record

    @staticmethod
    def get_error():
        """Gets the error status"""
        return Status._get_status('e')

    @staticmethod
    def get_completed():
        """Gets the completed status"""
        return Status._get_status('c')

    @staticmethod
    def get_processing():
        """Gets the processing status"""
        return Status._get_status('p')

    @staticmethod
    def get_queued():
        """Gets the queued status"""
        return Status._get_status('q')

    @staticmethod
    def get_skipped():
        """Gets the skipped status"""
        return Status._get_status('s')


class Software(models.Model):
    """
    Represents the software used to perform the reduction
    """
    name = models.CharField(max_length=100, blank=False, null=False)
    version = models.CharField(max_length=20, blank=False, null=False)

    def __str__(self):
        return f"{self.name}-{self.version}"


class ReductionRun(models.Model):
    """
    Table designed to link all table together. This represents a single reduction run that
    takes place at ISIS. Thus, this will store all the relevant data regarding that run.
    """
    # Integer fields
    run_number = models.IntegerField(blank=False, validators=[MinValueValidator(0)])
    run_version = models.IntegerField(blank=False, validators=[MinValueValidator(0)])
    started_by = models.IntegerField(null=True, blank=True)

    # Char fields
    run_description = models.CharField(max_length=200, blank=True)

    # Text fields
    admin_log = models.TextField(blank=True)
    graph = models.TextField(null=True, blank=True)
    message = models.TextField(null=True, blank=True)
    reduction_log = models.TextField(blank=True)
    reduction_host = models.TextField(default="", blank=True, verbose_name="Reduction hostname")
    # Scripts should be 100,000 chars or less. The DB supports up to 4GB strings here
    script = models.TextField(blank=False, validators=[MaxLengthValidator(100000)])

    # Date time fields
    created = models.DateTimeField(auto_now_add=True, blank=False)
    finished = models.DateTimeField(null=True, blank=True)
    last_updated = models.DateTimeField(auto_now=True, blank=False)
    retry_when = models.DateTimeField(null=True, blank=True)
    started = models.DateTimeField(null=True, blank=True)

    # Bool field
    hidden_in_failviewer = models.BooleanField(default=False)
    overwrite = models.NullBooleanField(default=True)

    # Foreign Keys
    experiment = models.ForeignKey(Experiment, blank=False, related_name='reduction_runs', on_delete=models.CASCADE)
    instrument = models.ForeignKey(Instrument, related_name='reduction_runs', null=True, on_delete=models.CASCADE)
    retry_run = models.ForeignKey('self', on_delete=models.SET_NULL, null=True, blank=True)
    status = models.ForeignKey(Status, blank=False, related_name='+', on_delete=models.CASCADE)
    # Allowed software field to be black in code line below. Issued opened (#852) to later
    # populate this field
    software = models.ForeignKey(Software,
                                 blank=True,
                                 related_name='reduction_runs',
                                 null=True,
                                 on_delete=models.CASCADE)

    def __str__(self):
        """
        Return str representation of reduction run based on run name if available else run number
        :return: str representation of ReductionRun
        """
        return f"{self.run_number}: {self.run_description}" if self.run_description else f"{self.run_number}"

    def title(self):
        """
        :return: An interface-friendly name that identifies this run using either
        run name or run version
        """
        if self.run_version > 0:
            if self.run_description:
                title = '%s - %s' % (self.run_number, self.run_description)
            else:
                title = '%s - %s' % (self.run_number, self.run_version)
        else:
            title = '%s' % self.run_number
        return title


class DataLocation(models.Model):
    """
    Represents the location at which the unreduced data is stored on disk
    """
    file_path = models.CharField(max_length=255)
    reduction_run = models.ForeignKey(ReductionRun, blank=False, related_name='data_location', on_delete=models.CASCADE)

    def __str__(self):
        """
        :return: str representation of file path
        """
        return f"{self.file_path}"


class ReductionLocation(models.Model):
    """
    Represents the location at which the reduced data is stored on disk
    """
    file_path = models.CharField(max_length=255)
    reduction_run = models.ForeignKey(ReductionRun,
                                      blank=False,
                                      related_name='reduction_location',
                                      on_delete=models.CASCADE)

    def __str__(self):
        """
        :return: str representation of ReductionLocation
        """
        return f"{self.file_path}"


class Setting(models.Model):
    """
    Represents additional settings options for the reduction run
    """
    name = models.CharField(max_length=50, blank=False)
    value = models.CharField(max_length=50)

    def __str__(self):
        """
        :return: str representation of Setting
        """
        return f"{self.name} = {self.value}"


class Notification(models.Model):
    """
    Represents possible notification messages regarding reduction runs
    """
    SEVERITY_CHOICES = (('i', 'info'), ('w', 'warning'), ('e', 'error'))

    message = models.CharField(max_length=255, blank=False)
    is_active = models.BooleanField(default=True)
    severity = models.CharField(max_length=1, choices=SEVERITY_CHOICES, default='i')
    is_staff_only = models.BooleanField(default=False)

    def __str__(self):
        """
        :return: str representation of Notification based on message
        """
        return f"Notification: {self.message}"

    def severity_verbose(self):
        """
        :return: the severity as its textual value
        """
        return dict(Notification.SEVERITY_CHOICES)[self.severity]


class OutputType(models.Model):
    """
    Represents the output types of file that can be output from a job
    This is an enum table
    """
    type = models.CharField(max_length=50, blank=False)


class Output(models.Model):
    """
    Represents the output of a reduction job (file path and type)
    """
    job = models.ForeignKey(ReductionRun, blank=False, related_name='output', on_delete=models.CASCADE)
    file_path = models.CharField(max_length=255, blank=False)
    type = models.ForeignKey(OutputType, blank=False, related_name='output', on_delete=models.CASCADE)



# pylint:skip-file
"""
Wrapper for the functionality for various installation and project setup commands
see:
    `python setup.py help`
for more details
"""
from os import path
from setuptools import setup, find_packages

# read the contents of the README file
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='autoreduce_db',
      version='22.0.0.dev6',
      description='ISIS Autoreduce',
      author='ISIS Autoreduction Team',
      url='https://github.com/ISISScientificComputing/autoreduce-db/',
      install_requires=['autoreduce-utils==22.0.0.dev4', 'Django==3.2.4'],
      packages=find_packages(),
      long_description=long_description,
      long_description_content_type='text/markdown')



# ############################################################################### #
# Autoreduction Repository : https://github.com/ISISScientificComputing/autoreduce
#
# Copyright &copy; 2020 ISIS Rutherford Appleton Laboratory UKRI
# SPDX - License - Identifier: GPL-3.0-or-later
# ############################################################################### #
"""
Definition of Variable classes used for the WebApp model
"""
from django.db import models
from autoreduce_db.reduction_viewer.models import Instrument, ReductionRun


class Variable(models.Model):
    """
    Generic model class that should be treated as abstract
    """
    name = models.CharField(max_length=50, blank=False)
    value = models.CharField(max_length=300, blank=True, null=True)
    type = models.CharField(max_length=50, blank=False)
    is_advanced = models.BooleanField(default=False)
    help_text = models.TextField(blank=True, null=True, default='')

    def sanitized_name(self):
        """
        Returns a HTMl-friendly name that can be used as element IDs or form input names
        """
        # pylint:disable=no-member
        return self.name.replace(' ', '-')


class InstrumentVariable(Variable):
    """
    Instrument specific variable class

    - Holds the IDs of the variables used for the instrument

    - Holds `start_run` for functionality to "Configure new runs" - e.g. variables starting from `start_run` will
      use the defaults that are queried with

    """
    instrument = models.ForeignKey(Instrument, on_delete=models.CASCADE)
    experiment_reference = models.IntegerField(blank=True, null=True)
    start_run = models.IntegerField(blank=True, null=True)
    tracks_script = models.BooleanField(default=False)

    # pylint:disable=no-member
    def __str__(self):
        return f"{self.instrument.name} - {self.name}=self.value"


class RunVariable(models.Model):
    """
    Run specific Variable class
    """
    variable = models.ForeignKey(Variable, related_name="runs", on_delete=models.CASCADE)
    reduction_run = models.ForeignKey(ReductionRun, related_name="run_variables", on_delete=models.CASCADE)
